@@ -22,7 +22,9 @@
 #ifndef _COMMON_H
 #define _COMMON_H
 
+#include <fstream>
 #include <climits>
+#include "dlist.h"
 
 typedef unsigned long long Size;
 
@@ -60,7 +62,10 @@ public:
 
 	bool isUpper() const { return ( 'A' <= key && key <= 'Z' ); }
 	bool isLower() const { return ( 'a' <= key && key <= 'z' ); }
-	bool isPrintable() const { return ( 32 <= key && key < 127 ); }
+	bool isPrintable() const
+	{
+	    return ( 7 <= key && key <= 13 ) || ( 32 <= key && key < 127 );
+	}
 
 	Key toUpper() const
 		{ return Key( 'A' + ( key - 'a' ) ); }
@@ -94,6 +99,7 @@ struct HostType
 {
 	char *data1;
 	char *data2;
+	char *internalName;
 	bool isSigned;
 	long long minVal;
 	long long maxVal;
@@ -102,27 +108,29 @@ struct HostType
 
 struct HostLang
 {
+	/* Target language. */
+	enum Lang
+	{
+		C, D, Java, Ruby
+	};
+
+	Lang lang;
 	HostType *hostTypes;
 	int numHostTypes;
 	HostType *defaultAlphType;
 	bool explicitUnsigned;
 };
 
-
-/* Target language. */
-enum HostLangType
-{
-	CCode,
-	DCode,
-	JavaCode
-};
-
 extern HostLang *hostLang;
-extern HostLangType hostLangType;
 
 extern HostLang hostLangC;
 extern HostLang hostLangD;
 extern HostLang hostLangJava;
+extern HostLang hostLangRuby;
+
+HostType *findAlphType( char *s1 );
+HostType *findAlphType( char *s1, char *s2 );
+HostType *findAlphTypeInternal( char *s1 );
 
 /* An abstraction of the key operators that manages key operations such as
  * comparison and increment according the signedness of the key. */
@@ -268,4 +276,38 @@ inline Key operator/(const Key key1, const Key key2)
 	return key1.key / key2.key;
 }
 
+/* Filter on the output stream that keeps track of the number of lines
+ * output. */
+class output_filter : public std::filebuf
+{
+public:
+	output_filter( char *fileName ) : fileName(fileName), line(1) { }
+
+	virtual int sync();
+	virtual std::streamsize xsputn(const char* s, std::streamsize n);
+
+	char *fileName;
+	int line;
+};
+
+char *findFileExtension( char *stemFile );
+char *fileNameFromStem( char *stemFile, char *suffix );
+
+struct Export
+{
+	Export( char *name, Key key )
+		: name(name), key(key) {}
+
+	char *name;
+	Key key;
+
+	Export *prev, *next;
+};
+
+typedef DList<Export> ExportList;
+
+struct exit_object { };
+extern exit_object endp;
+void operator<<( std::ostream &out, exit_object & );
+
 #endif /* _COMMON_H */
