@@ -82,7 +82,7 @@ void FsmAp::detachFromInList( StateAp *from, StateAp *to,
 /* Attach states on the default transition, range list or on out/in list key.
  * First makes a new transition. If there is already a transition out from
  * fromState on the default, then will assertion fail. */
-TransAp *FsmAp::attachNewTrans( StateAp *from, StateAp *to, long lowKey, long highKey )
+TransAp *FsmAp::attachNewTrans( StateAp *from, StateAp *to, Key lowKey, Key highKey )
 {
 	/* Make the new transition. */
 	TransAp *retVal = new TransAp();
@@ -190,11 +190,10 @@ void FsmAp::detachState( StateAp *state )
 
 /* Duplicate a transition. Makes a new transition that is attached to the same
  * dest as srcTrans. The new transition has functions and priority taken from
- * srcTrans. If leavingFsm is set, then outPrior and outFuncs are taken from
- * the from state. Used for merging a transition in to a free spot. The trans
- * can just be dropped in. It does not conflict with an existing trans and need
+ * srcTrans. Used for merging a transition in to a free spot. The trans can
+ * just be dropped in. It does not conflict with an existing trans and need
  * not be crossed. Returns the new transition. */
-TransAp *FsmAp::dupTrans( StateAp *from, TransAp *srcTrans, bool leavingFsm )
+TransAp *FsmAp::dupTrans( StateAp *from, TransAp *srcTrans )
 {
 	/* Make a new transition. */
 	TransAp *newTrans = new TransAp();
@@ -205,21 +204,13 @@ TransAp *FsmAp::dupTrans( StateAp *from, TransAp *srcTrans, bool leavingFsm )
 	/* Call the user callback to add in the original source transition. */
 	addInTrans( newTrans, srcTrans );
 
-	/* If the transition being duplicated is not an error transition and we
-	 * are leaving, then add in the from state's out data. This will pickup
-	 * any out functions or and/or set the priority from the out prior. This
-	 * must happen after the adding in of the trans, so that out priority
-	 * overrides the priority from the source trans. */
-	if ( newTrans->toState != 0 && leavingFsm )
-		leavingFromState( newTrans, from );
-
 	return newTrans;
 }
 
 /* In crossing, src trans and dest trans both go to existing states. Make one
  * state from the sets of states that src and dest trans go to. */
 TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from,
-			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
+			TransAp *destTrans, TransAp *srcTrans )
 {
 	/* The priorities are equal. We must merge the transitions. Does the
 	 * existing trans go to the state we are to attach to? ie, are we to
@@ -288,7 +279,7 @@ TransAp *FsmAp::fsmAttachStates( MergeData &md, StateAp *from,
 /* Two transitions are to be crossed, handle the possibility of either going
  * to the error state. */
 TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from,
-			TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
+			TransAp *destTrans, TransAp *srcTrans )
 {
 	TransAp *retTrans = 0;
 	if ( destTrans->toState == 0 && srcTrans->toState == 0 ) {
@@ -302,9 +293,6 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from,
 		attachTrans( from, srcTrans->toState, destTrans );
 		addInTrans( destTrans, srcTrans );
 		retTrans = destTrans;
-
-		if ( leavingFsm )
-			leavingFromState( retTrans, from );
 	}
 	else if ( srcTrans->toState == 0 ) {
 		/* Dest goes somewhere but src doesn't, just add it it in. */
@@ -313,14 +301,9 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from,
 	}
 	else {
 		/* Both go somewhere, run the actual cross. */
-		retTrans = fsmAttachStates( md, from, destTrans, 
-				srcTrans, leavingFsm );
-
-		if ( leavingFsm )
-			leavingFromState( retTrans, from );
+		retTrans = fsmAttachStates( md, from, destTrans, srcTrans );
 	}
 
-
 	return retTrans;
 }
 
@@ -328,22 +311,17 @@ TransAp *FsmAp::mergeTrans( MergeData &md, StateAp *from,
  * src is ignored. If src is higher priority than dest, then src overwrites dest. If
  * the priorities are equal, then they are merged. */
 TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from,
-		TransAp *destTrans, TransAp *srcTrans, bool leavingFsm )
+		TransAp *destTrans, TransAp *srcTrans )
 {
 	TransAp *retTrans;
 
-	/* Compute the priority table of the source transition. */
-	PriorTable srcPrior = srcTrans->priorTable;
-	if ( leavingFsm )
-		srcPrior.setPriors( from->outPriorTable );
-
 	/* Compare the priority of the dest and src transitions. */
-	int compareRes = comparePrior( destTrans->priorTable, srcPrior );
+	int compareRes = comparePrior( destTrans->priorTable, srcTrans->priorTable );
 	if ( compareRes < 0 ) {
 		/* Src trans has a higher priority than dest, src overwrites dest.
 		 * Detach dest and return a copy of src. */
 		detachTrans( from, destTrans->toState, destTrans );
-		retTrans = dupTrans( from, srcTrans, leavingFsm );
+		retTrans = dupTrans( from, srcTrans );
 	}
 	else if ( compareRes > 0 ) {
 		/* The dest trans has a higher priority, use dest. */
@@ -351,124 +329,65 @@ TransAp *FsmAp::crossTransitions( MergeData &md, StateAp *from,
 	}
 	else {
 		/* Src trans and dest trans have the same priority, they must be merged. */
-		retTrans = mergeTrans( md, from, destTrans, 
-				srcTrans, leavingFsm );
+		retTrans = mergeTrans( md, from, destTrans, srcTrans );
 	}
 
 	/* Return the transition that resulted from the cross. */
 	return retTrans;
 }
 
-/* When copying out transitions, a transition exists only in the dest out list.
- * Since the key is not in src, we must consider a default transition.  Dest
- * may get crossed with it. */
-TransAp *FsmAp::keyInDestEl( MergeData &md, StateAp *dest, StateAp *src, 
-		 NextTrans *destTel, bool leavingFsm )
+/* Copy the transitions in srcList to the outlist of dest. The srcList should
+ * not be the outList of dest, otherwise you would be copying the contents of
+ * srcList into itself as it's iterated: bad news. */
+void FsmAp::outTransCopy( MergeData &md, StateAp *dest, TransAp *srcList )
 {
-	TransAp *retTrans = 0;
-	retTrans = destTel->trans;
-	return retTrans;
-}
-
-/* When copying out transitions, a transition exists only in the src out list.
- * Since the key is not in dest, we must consider a default transition. Src may
- * get crossed with one of those. */
-TransAp *FsmAp::keyInSrcEl( MergeData &md, StateAp *dest, StateAp *src, 
-		NextTrans *srcTel, bool leavingFsm )
-{
-	TransAp *retTrans = 0;
-	retTrans = dupTrans( dest, srcTel->trans, leavingFsm );
-	return retTrans;
-}
-
-void FsmAp::changeRangeLowerKey( TransAp *trans, long oldKey, 
-		long newKey )
-{
-	/* Be certain that the the we are changing from the correct old key and
-	 * set the new key. */
-	assert( keyOps->eq( trans->lowKey, oldKey ) );
-	trans->lowKey = newKey;
-}
-
-/* Append a transition to a destination list. If any gaps are found then
- * reports that by setting covers = false. */
-void FsmAp::appendTrans( TransList &destList, TransAp *trans, bool &covers )
-{
-	if ( destList.length() == 0 ) {
-		if ( keyOps->lt( keyOps->lowKey, trans->lowKey ) )
-			covers = false;
-	}
-	else {
-		long lastHigh = destList.tail->highKey;
-		keyOps->inc( lastHigh );
-		if ( keyOps->lt( lastHigh, trans->lowKey ) )
-			covers = false;
-	}
-	destList.append( trans );
-}
-
-/* Copy the transitions in srcList to the outlist of dest. If srcList comes from
- * a final state and out transitions should be copied in then leavingFsm can be
- * set true. SrcList should not be the outList of dest, otherwise you would
- * be copying the contents of srcList into itself as it's iterated: bad news. */
-void FsmAp::outTransCopy( MergeData &md, StateAp *dest, StateAp *src, bool leavingFsm )
-{
-	/* For making destination lists. */
-	TransEl newTel;
-	bool covers = true;
-
 	/* The destination list. */
 	TransList destList;
 
 	/* Set up an iterator to stop at breaks. */
-	PairIter outPair( dest, src, true );
-	while ( ! outPair.end() ) {
+	PairIter<TransAp> outPair( dest->outList.head, srcList );
+	for ( ; !outPair.end(); outPair++ ) {
 		switch ( outPair.userState ) {
-		case PairIter::RangeInS1: {
-			/* Dest range may get crossed with src's default transition. */
-			TransAp *newTrans = keyInDestEl( md, dest, src, &outPair.s1Tel, leavingFsm );
-
-			/* Set up the transition's keys and append to the dest list. */
-			newTrans->lowKey = outPair.s1Tel.lowKey;
-			newTrans->highKey = outPair.s1Tel.highKey;
-			appendTrans( destList, newTrans, covers );
+		case RangeInS1: {
+			/* The pair iter is the authority on the keys. It may have needed
+			 * to break the dest range. */
+			TransAp *destTrans = outPair.s1Tel.trans;
+			destTrans->lowKey = outPair.s1Tel.lowKey;
+			destTrans->highKey = outPair.s1Tel.highKey;
+			destList.append( destTrans );
 			break;
 		}
-		case PairIter::RangeInS2: {
+		case RangeInS2: {
 			/* Src range may get crossed with dest's default transition. */
-			TransAp *newTrans = keyInSrcEl( md, dest, src, &outPair.s2Tel, leavingFsm );
+			TransAp *newTrans = dupTrans( dest, outPair.s2Tel.trans );
 
 			/* Set up the transition's keys and append to the dest list. */
 			newTrans->lowKey = outPair.s2Tel.lowKey;
 			newTrans->highKey = outPair.s2Tel.highKey;
-			appendTrans( destList, newTrans, covers );
+			destList.append( newTrans );
 			break;
 		}
-		case PairIter::RangeOverlap: {
+		case RangeOverlap: {
 			/* Exact overlap, cross them. */
 			TransAp *newTrans = crossTransitions( md, dest,
-				outPair.s1Tel.trans, outPair.s2Tel.trans, leavingFsm );
+				outPair.s1Tel.trans, outPair.s2Tel.trans );
 
 			/* Set up the transition's keys and append to the dest list. */
 			newTrans->lowKey = outPair.s1Tel.lowKey;
 			newTrans->highKey = outPair.s1Tel.highKey;
-			appendTrans( destList, newTrans, covers );
+			destList.append( newTrans );
 			break;
 		}
-		case PairIter::BreakS1: {
+		case BreakS1: {
 			/* Since we are always writing to the dest trans, the dest needs
-			 * to be coppied when it is broken. The copy goes into the first
-			 * half of the break because that is the value we have access to.
-			 * */
-			outPair.s1Tel.trans = dupTrans( dest,
-					outPair.s1Tel.trans, false );
+			 * to be copied when it is broken. The copy goes into the first
+			 * half of the break to "break it off". */
+			outPair.s1Tel.trans = dupTrans( dest, outPair.s1Tel.trans );
 			break;
 		}
-		case PairIter::BreakS2:
+		case BreakS2:
 			break;
 		}
-
-		outPair++;
 	}
 
 	/* Abandon the old outList and transfer destList into it. */
