@@ -26,21 +26,6 @@
 #include <iostream>
 using namespace std;
 
-/* Return and re-entry for the co-routine iterators. This should ALWAYS be
- * used inside of a block. */
-#define CO_RETURN(label) \
-	itState = label; \
-	return; \
-	entry##label: backIn = true
-
-/* Return and re-entry for the co-routine iterators. This should ALWAYS be
- * used inside of a block. */
-#define CO_RETURN2(label, uState) \
-	itState = label; \
-	userState = uState; \
-	return; \
-	entry##label: backIn = true
-
 /* Construct a mark index for a specified number of states. Must new up
  * an array that is states^2 in size. */
 MarkIndex::MarkIndex( int states ) : numStates(states)
@@ -83,49 +68,6 @@ bool MarkIndex::isPairMarked(int state1, int state2)
 	return array[pos];
 }
 
-/*
- * FsmKeySet
- */
-
-/* Return a pointer to the item in the set on success. */
-long *KeySet::insert( long key )
-{
-	long *lower, *upper;
-	int dataLen = size();
-	if ( dataLen == 0 ) {
-		lower = data;
-		goto insert;
-	}
-
-	lower = data;
-	upper = data + dataLen - 1;
-	while ( true ) {
-		/* Not found, insert location is lower. */
-		if ( upper < lower )
-			goto insert;
-
-		long *mid = lower + ((upper-lower)>>1);
-		if ( keyOps->lt( key, *mid ) )
-			upper = mid - 1;
-		else if ( keyOps->gt( key, *mid ) )
-			lower = mid + 1;
-		else {
-			/* The key was found, return failure. */
-			return 0;
-		}
-	}
-insert:
-	/* Get the insert pos. */
-	int insertPos = lower - data;
-
-	/* Do the insert. After makeRawSpaceFor, lower pointer is no good. */
-	makeRawSpaceFor( insertPos, 1 );
-	data[insertPos] = key;
-
-	/* Return the new element. */
-	return data + insertPos;
-}
-
 /* Create a new fsm state. State has not out transitions or in transitions, not
  * out out transition data and not number. */
 StateAp::StateAp()
@@ -139,7 +81,7 @@ StateAp::StateAp()
 	epsilonTrans(),
 
 	/* Conditions. */
-	stateCondDict(),
+	stateCondList(),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -158,6 +100,7 @@ StateAp::StateAp()
 	toStateActionTable(),
 	fromStateActionTable(),
 	outActionTable(),
+	outCondSet(),
 	errActionTable(),
 	eofActionTable()
 {
@@ -177,8 +120,8 @@ StateAp::StateAp(const StateAp &other)
 	entryIds(other.entryIds),
 	epsilonTrans(other.epsilonTrans),
 
-	/* Copy in the elements of the avl tree-based dict. */
-	stateCondDict( other.stateCondDict ),
+	/* Copy in the elements of the conditions. */
+	stateCondList( other.stateCondList ),
 
 	/* No transitions in from other states. */
 	foreignInTrans(0),
@@ -197,6 +140,7 @@ StateAp::StateAp(const StateAp &other)
 	toStateActionTable(other.toStateActionTable),
 	fromStateActionTable(other.fromStateActionTable),
 	outActionTable(other.outActionTable),
+	outCondSet(other.outCondSet),
 	errActionTable(other.errActionTable),
 	eofActionTable(other.eofActionTable)
 {
@@ -243,34 +187,33 @@ int ApproxCompare::compare( const StateAp *state1 , const StateAp *state2 )
 		return compareRes;
 
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, false );
-	while ( ! outPair.end() ) {
+	PairIter<TransAp> outPair( state1->outList.head, state2->outList.head );
+	for ( ; !outPair.end(); outPair++ ) {
 		switch ( outPair.userState ) {
 
-		case PairIter::RangeInS1:
+		case RangeInS1:
 			compareRes = FsmAp::compareFullPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeInS2:
+		case RangeInS2:
 			compareRes = FsmAp::compareFullPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeOverlap:
+		case RangeOverlap:
 			compareRes = FsmAp::compareFullPtr( 
 					outPair.s1Tel.trans, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::BreakS1:
-		case PairIter::BreakS2:
+		case BreakS1:
+		case BreakS2:
 			break;
 		}
-		outPair++;
 	}
 
 	/* Got through the entire state comparison, deem them equal. */
@@ -299,35 +242,58 @@ int InitPartitionCompare::compare( const StateAp *state1 , const StateAp *state2
 	if ( compareRes != 0 )
 		return compareRes;
 
-	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, false );
-	while ( ! outPair.end() ) {
+	/* Use a pair iterator to test the condition pairs. */
+	PairIter<StateCond> condPair( state1->stateCondList.head, state2->stateCondList.head );
+	for ( ; !condPair.end(); condPair++ ) {
+		switch ( condPair.userState ) {
+		case RangeInS1:
+			return 1;
+		case RangeInS2:
+			return -1;
+
+		case RangeOverlap: {
+			CondSpace *condSpace1 = condPair.s1Tel.trans->condSpace;
+			CondSpace *condSpace2 = condPair.s2Tel.trans->condSpace;
+			if ( condSpace1 < condSpace2 )
+				return -1;
+			else if ( condSpace1 > condSpace2 )
+				return 1;
+			break;
+		}
+		case BreakS1:
+		case BreakS2:
+			break;
+		}
+	}
+
+	/* Use a pair iterator to test the transition pairs. */
+	PairIter<TransAp> outPair( state1->outList.head, state2->outList.head );
+	for ( ; !outPair.end(); outPair++ ) {
 		switch ( outPair.userState ) {
 
-		case PairIter::RangeInS1:
+		case RangeInS1:
 			compareRes = FsmAp::compareDataPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeInS2:
+		case RangeInS2:
 			compareRes = FsmAp::compareDataPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeOverlap:
+		case RangeOverlap:
 			compareRes = FsmAp::compareDataPtr( 
 					outPair.s1Tel.trans, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::BreakS1:
-		case PairIter::BreakS2:
+		case BreakS1:
+		case BreakS2:
 			break;
 		}
-		outPair++;
 	}
 
 	return 0;
@@ -339,34 +305,33 @@ int PartitionCompare::compare( const StateAp *state1, const StateAp *state2 )
 	int compareRes;
 
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, false );
-	while ( ! outPair.end() ) {
+	PairIter<TransAp> outPair( state1->outList.head, state2->outList.head );
+	for ( ; !outPair.end(); outPair++ ) {
 		switch ( outPair.userState ) {
 
-		case PairIter::RangeInS1:
+		case RangeInS1:
 			compareRes = FsmAp::comparePartPtr( outPair.s1Tel.trans, 0 );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeInS2:
+		case RangeInS2:
 			compareRes = FsmAp::comparePartPtr( 0, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::RangeOverlap:
+		case RangeOverlap:
 			compareRes = FsmAp::comparePartPtr( 
 					outPair.s1Tel.trans, outPair.s2Tel.trans );
 			if ( compareRes != 0 )
 				return compareRes;
 			break;
 
-		case PairIter::BreakS1:
-		case PairIter::BreakS2:
+		case BreakS1:
+		case BreakS2:
 			break;
 		}
-		outPair++;
 	}
 
 	return 0;
@@ -377,229 +342,35 @@ bool MarkCompare::shouldMark( MarkIndex &markIndex, const StateAp *state1,
 			const StateAp *state2 )
 {
 	/* Use a pair iterator to get the transition pairs. */
-	PairIter outPair( state1, state2, false );
-	while ( ! outPair.end() ) {
+	PairIter<TransAp> outPair( state1->outList.head, state2->outList.head );
+	for ( ; !outPair.end(); outPair++ ) {
 		switch ( outPair.userState ) {
 
-		case PairIter::RangeInS1:
+		case RangeInS1:
 			if ( FsmAp::shouldMarkPtr( markIndex, outPair.s1Tel.trans, 0 ) )
 				return true;
 			break;
 
-		case PairIter::RangeInS2:
+		case RangeInS2:
 			if ( FsmAp::shouldMarkPtr( markIndex, 0, outPair.s2Tel.trans ) )
 				return true;
 			break;
 
-		case PairIter::RangeOverlap:
+		case RangeOverlap:
 			if ( FsmAp::shouldMarkPtr( markIndex,
 					outPair.s1Tel.trans, outPair.s2Tel.trans ) )
 				return true;
 			break;
 
-		case PairIter::BreakS1:
-		case PairIter::BreakS2:
+		case BreakS1:
+		case BreakS2:
 			break;
 		}
-		outPair++;
 	}
 
 	return false;
 }
 
-/* Init the iterator by advancing to the first item. */
-PairIter::PairIter( const StateAp *state1, const StateAp *state2, bool wantBreaks )
-:
-	state1(state1),
-	state2(state2),
-	itState(Begin),
-	wantBreaks(wantBreaks)
-{
-	findNext();
-}
-
-/* Advance to the next transition. When returns, trans points to the next
- * transition, unless there are no more, in which case end() returns true. */
-void PairIter::findNext()
-{
-	/* This variable is used in dummy statements that follow the entry
-	 * goto labels. The compiler needs some statement to follow the label. */
-	bool backIn;
-
-	/* Jump into the iterator routine base on the iterator state. */
-	switch ( itState ) {
-		case Begin:              goto entryBegin;
-		case ConsumeS1Range:     goto entryConsumeS1Range;
-		case ConsumeS2Range:     goto entryConsumeS2Range;
-		case OnlyInS1Range:      goto entryOnlyInS1Range;
-		case OnlyInS2Range:      goto entryOnlyInS2Range;
-		case S1SticksOut:        goto entryS1SticksOut;
-		case S1SticksOutBreak:   goto entryS1SticksOutBreak;
-		case S2SticksOut:        goto entryS2SticksOut;
-		case S2SticksOutBreak:   goto entryS2SticksOutBreak;
-		case S1DragsBehind:      goto entryS1DragsBehind;
-		case S1DragsBehindBreak: goto entryS1DragsBehindBreak;
-		case S2DragsBehind:      goto entryS2DragsBehind;
-		case S2DragsBehindBreak: goto entryS2DragsBehindBreak;
-		case ExactOverlap:       goto entryExactOverlap;
-		case End:                goto entryEnd;
-	}
-
-entryBegin:
-	/* Set up the next structs at the head of the transition lists. */
-	s1Tel.set( state1->outList.head );
-	s2Tel.set( state2->outList.head );
-
-	/* Concurrently scan both out ranges. */
-	while ( true ) {
-		if ( s1Tel.trans == 0 ) {
-			/* We are at the end of state1's ranges. Process the rest of
-			 * state2's ranges. */
-			while ( s2Tel.trans != 0 ) {
-				/* Range is only in s2. */
-				CO_RETURN2( ConsumeS2Range, RangeInS2 );
-				s2Tel.increment();
-			}
-			break;
-		}
-		else if ( s2Tel.trans == 0 ) {
-			/* We are at the end of state2's ranges. Process the rest of
-			 * state1's ranges. */
-			while ( s1Tel.trans != 0 ) {
-				/* Range is only in s1. */
-				CO_RETURN2( ConsumeS1Range, RangeInS1 );
-				s1Tel.increment();
-			}
-			break;
-		}
-		/* Both state1's and state2's transition elements are good.
-		 * The signiture of no overlap is a back key being in front of a
-		 * front key. */
-		else if ( keyOps->lt( s1Tel.highKey, s2Tel.lowKey ) ) {
-			/* A range exists in state1 that does not overlap with state2. */
-			CO_RETURN2( OnlyInS1Range, RangeInS1 );
-			s1Tel.increment();
-		}
-		else if ( keyOps->lt( s2Tel.highKey, s1Tel.lowKey ) ) {
-			/* A range exists in state2 that does not overlap with state1. */
-			CO_RETURN2( OnlyInS2Range, RangeInS2 );
-			s2Tel.increment();
-		}
-		/* There is overlap, must mix the ranges in some way. */
-		else if ( keyOps->lt( s1Tel.lowKey, s2Tel.lowKey ) ) {
-			/* Range from state1 sticks out front. Must break it into
-			 * non-overlaping and overlaping segments. */
-			bottomLow = s2Tel.lowKey;
-			bottomHigh = s1Tel.highKey;
-			s1Tel.highKey = s2Tel.lowKey;
-			keyOps->dec( s1Tel.highKey );
-			bottomTrans = s1Tel.trans;
-
-			if ( wantBreaks ) {
-				/* Notify the caller that we are breaking s1. This gives them a
-				 * chance to duplicate s1Tel[0,1].value. */
-				CO_RETURN2( S1SticksOutBreak, BreakS1 );
-			}
-
-			/* Broken off range is only in s1. */
-			CO_RETURN2( S1SticksOut, RangeInS1 );
-
-			/* Advance over the part sticking out front. */
-			s1Tel.lowKey = bottomLow;
-			s1Tel.highKey = bottomHigh;
-			s1Tel.trans = bottomTrans;
-		}
-		else if ( keyOps->lt( s2Tel.lowKey, s1Tel.lowKey ) ) {
-			/* Range from state2 sticks out front. Must break it into
-			 * non-overlaping and overlaping segments. */
-			bottomLow = s1Tel.lowKey;
-			bottomHigh = s2Tel.highKey;
-			s2Tel.highKey = s1Tel.lowKey;
-			keyOps->dec( s2Tel.highKey );
-			bottomTrans = s2Tel.trans;
-
-			if ( wantBreaks ) {
-				/* Notify the caller that we are breaking s2. This gives them a
-				 * chance to duplicate s2Tel[0,1].value. */
-				CO_RETURN2( S2SticksOutBreak, BreakS2 );
-			}
-
-			/* Broken off range is only in s2. */
-			CO_RETURN2( S2SticksOut, RangeInS2 );
-
-			/* Advance over the part sticking out front. */
-			s2Tel.lowKey = bottomLow;
-			s2Tel.highKey = bottomHigh;
-			s2Tel.trans = bottomTrans;
-		}
-		/* Low ends are even. Are the high ends even? */
-		else if ( keyOps->lt( s1Tel.highKey, s2Tel.highKey ) ) {
-			/* Range from state2 goes longer than the range from state1. We
-			 * must break the range from state2 into an evenly overlaping
-			 * segment. */
-			bottomLow = s1Tel.highKey;
-			keyOps->inc( bottomLow );
-			bottomHigh = s2Tel.highKey;
-			s2Tel.highKey = s1Tel.highKey;
-			bottomTrans = s2Tel.trans;
-
-			if ( wantBreaks ) {
-				/* Notify the caller that we are breaking s2. This gives them a
-				 * chance to duplicate s2Tel[0,1].value. */
-				CO_RETURN2( S2DragsBehindBreak, BreakS2 );
-			}
-
-			/* Breaking s2 produces exact overlap. */
-			CO_RETURN2( S2DragsBehind, RangeOverlap );
-
-			/* Advance over the front we just broke off of range 2. */
-			s2Tel.lowKey = bottomLow;
-			s2Tel.highKey = bottomHigh;
-			s2Tel.trans = bottomTrans;
-
-			/* Advance over the entire s1Tel. We have consumed it. */
-			s1Tel.increment();
-		}
-		else if ( keyOps->lt( s2Tel.highKey, s1Tel.highKey ) ) {
-			/* Range from state1 goes longer than the range from state2. We
-			 * must break the range from state1 into an evenly overlaping
-			 * segment. */
-			bottomLow = s2Tel.highKey;
-			keyOps->inc( bottomLow );
-			bottomHigh = s1Tel.highKey;
-			s1Tel.highKey = s2Tel.highKey;
-			bottomTrans = s1Tel.trans;
-
-			if ( wantBreaks ) {
-				/* Notify the caller that we are breaking s1. This gives them a
-				 * chance to duplicate s2Tel[0,1].value. */
-				CO_RETURN2( S1DragsBehindBreak, BreakS1 );
-			}
-
-			/* Breaking s1 produces exact overlap. */
-			CO_RETURN2( S1DragsBehind, RangeOverlap );
-
-			/* Advance over the front we just broke off of range 1. */
-			s1Tel.lowKey = bottomLow;
-			s1Tel.highKey = bottomHigh;
-			s1Tel.trans = bottomTrans;
-
-			/* Advance over the entire s2Tel. We have consumed it. */
-			s2Tel.increment();
-		}
-		else {
-			/* There is an exact overlap. */
-			CO_RETURN2( ExactOverlap, RangeOverlap );
-
-			s1Tel.increment();
-			s2Tel.increment();
-		}
-	}
-
-	/* Done, go into end state. */
-	CO_RETURN( End );
-}
-
 /*
  * Transition Comparison.
  */
