@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001-2007 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -29,26 +29,23 @@
 
 /* Parsing. */
 #include "ragel.h"
-#include "rlparse.h"
-#include "parsetree.h"
+#include "rlscan.h"
 
 /* Parameters and output. */
 #include "pcheck.h"
 #include "vector.h"
 #include "version.h"
+#include "common.h"
 
-#include "common.cpp"
-
+using std::istream;
 using std::ostream;
+using std::ifstream;
 using std::ofstream;
+using std::cin;
 using std::cout;
 using std::cerr;
 using std::endl;
 
-/* Io globals. */
-ostream *outStream = 0;
-char *outputFileName = 0;
-
 /* Controls minimization. */
 MinimizeLevel minimizeLevel = MinimizePartition2;
 MinimizeOpt minimizeOpt = MinimizeMostOps;
@@ -75,12 +72,13 @@ void usage()
 "   -l                   Minimize after most operations (default)\n"
 "   -e                   Minimize after every operation\n"
 "machine selection:\n"
-"   -S <spec>            FSM specification to output for -V\n"
-"   -M <machine>         Machine definition/instantiation to output for -V\n"
+"   -S <spec>            FSM specification to output (for rlgen-dot)\n"
+"   -M <machine>         Machine definition/instantiation to output (for rlgen-dot)\n"
 "host language:\n"
 "   -C                   The host language is C, C++, Obj-C or Obj-C++ (default)\n"
 "   -D                   The host language is D\n"
 "   -J                   The host language is Java\n"
+"   -R                   The host language is Ruby\n"
 	;	
 }
 
@@ -88,84 +86,34 @@ void usage()
 void version()
 {
 	cout << "Ragel State Machine Compiler version " VERSION << " " PUBDATE << endl <<
-			"Copyright (c) 2001-2006 by Adrian Thurston" << endl;
+			"Copyright (c) 2001-2007 by Adrian Thurston" << endl;
 }
 
-/* Global parse data pointer. */
-extern InputData *id;
-
 /* Total error count. */
 int gblErrorCount = 0;
 
-/* Print the opening to a program error, then return the error stream. */
-ostream &error()
-{
-	/* Keep the error count. */
-	if ( id != 0 && id->pd != 0 )
-		id->pd->errorCount += 1;
-	gblErrorCount += 1;
-	
-	cerr << PROGNAME ": ";
-	return cerr;
-}
-
-/* Print the opening to an error in the input, then return the error ostream. */
-ostream &error( const YYLTYPE &loc )
-{
-	/* Keep the error count. */
-	if ( id->pd != 0 )
-		id->pd->errorCount += 1;
-	gblErrorCount += 1;
-
-	cerr << id->fileName << ":" << loc.first_line << ":" << 
-			loc.first_column << ": ";
-	return cerr;
-}
-
-/* Print the opening to an error in the input, then return the error ostream. */
-ostream &error( const InputLoc &loc )
-{
-	/* Keep the error count. */
-	if ( id->pd != 0 )
-		id->pd->errorCount += 1;
-	gblErrorCount += 1;
-
-	cerr << id->fileName << ":" << loc.line << ":" << 
-			loc.col << ": ";
-	return cerr;
-}
-
-ostream &error( int first_line, int first_column )
-{
-	/* Keep the error count. */
-	if ( id->pd != 0 )
-		id->pd->errorCount += 1;
-	gblErrorCount += 1;
-
-	cerr << id->fileName << ":" << first_line << ":" << first_column << ": ";
-	return cerr;
-}
-
-/* Print the opening to a warning, then return the error ostream. */
-ostream &warning( )
-{
-	cerr << id->fileName << ": warning: ";
-	return cerr;
-}
-
 /* Print the opening to a warning in the input, then return the error ostream. */
 ostream &warning( const InputLoc &loc )
 {
-	cerr << id->fileName << ":" << loc.line << ":" << 
+	assert( loc.fileName != 0 );
+	cerr << loc.fileName << ":" << loc.line << ":" << 
 			loc.col << ": warning: ";
 	return cerr;
 }
 
-/* Print the opening to a warning in the input, then return the error ostream. */
-std::ostream &warning( int first_line, int first_column )
+/* Print the opening to a program error, then return the error stream. */
+ostream &error()
 {
-	cerr << id->fileName << ":" << first_line << ":" << 
-			first_column << ": warning: ";
+	gblErrorCount += 1;
+	cerr << PROGNAME ": ";
+	return cerr;
+}
+
+ostream &error( const InputLoc &loc )
+{
+	gblErrorCount += 1;
+	assert( loc.fileName != 0 );
+	cerr << loc.fileName << ":" << loc.line << ": ";
 	return cerr;
 }
 
@@ -182,8 +130,9 @@ void escapeLineDirectivePath( std::ostream &out, char *path )
 /* Main, process args and call yyparse to start scanning input. */
 int main(int argc, char **argv)
 {
-	ParamCheck pc("o:nmleabjkS:M:CDJvHh?-:s", argc, argv);
+	ParamCheck pc("o:nmleabjkS:M:CDJRvHh?-:s", argc, argv);
 	char *inputFileName = 0;
+	char *outputFileName = 0;
 
 	while ( pc.check() ) {
 		switch ( pc.state ) {
@@ -253,17 +202,17 @@ int main(int argc, char **argv)
 
 			/* Host language types. */
 			case 'C':
-				hostLangType = CCode;
 				hostLang = &hostLangC;
 				break;
 			case 'D':
-				hostLangType = DCode;
 				hostLang = &hostLangD;
 				break;
 			case 'J':
-				hostLangType = JavaCode;
 				hostLang = &hostLangJava;
 				break;
+			case 'R':
+				hostLang = &hostLangRuby;
+				break;
 
 			/* Version and help. */
 			case 'v':
@@ -322,15 +271,17 @@ int main(int argc, char **argv)
 	}
 
 	/* Open the input file for reading. */
+	istream *inStream;
 	if ( inputFileName != 0 ) {
-		yyin = fopen( inputFileName, "rt" );
-
-		if ( yyin == 0 )
+		/* Open the input file for reading. */
+		ifstream *inFile = new ifstream( inputFileName );
+		inStream = inFile;
+		if ( ! inFile->is_open() )
 			error() << "could not open " << inputFileName << " for reading" << endl;
 	}
 	else {
 		inputFileName = "<stdin>";
-		yyin = stdin;
+		inStream = &cin;
 	}
 
 
@@ -339,25 +290,26 @@ int main(int argc, char **argv)
 		exit(1);
 
 	std::ostringstream outputBuffer;
-	outStream = &outputBuffer;
-
-	/* Create the input data for the top level file. */
-	id = new InputData( inputFileName, 0, 0 );
 
 	if ( machineSpec == 0 && machineName == 0 )
-		*outStream << "<host line=\"1\" col=\"1\">";
+		outputBuffer << "<host line=\"1\" col=\"1\">";
 
-	/* Parse the input! */
-	yyparse();
+	Scanner scanner( inputFileName, *inStream, outputBuffer, 0, 0, 0, false );
+	scanner.do_scan();
 
-	if ( machineSpec == 0 && machineName == 0 )
-		*outStream << "</host>\n";
+	/* Finished, final check for errors.. */
+	if ( gblErrorCount > 0 )
+		return 1;
+	
+	/* Now send EOF to all parsers. */
+	terminateAllParsers();
 
 	/* Finished, final check for errors.. */
 	if ( gblErrorCount > 0 )
 		return 1;
 
-	checkMachines();
+	if ( machineSpec == 0 && machineName == 0 )
+		outputBuffer << "</host>\n";
 
 	if ( gblErrorCount > 0 )
 		return 1;
