@@ -31,19 +31,21 @@
 #include "parsetree.h"
 #include "mergesort.h"
 #include "xmlcodegen.h"
+#include "version.h"
 
 using namespace std;
 
-char machineMain[] = "main";
+char mainMachine[] = "main";
 
-InputLoc::InputLoc( const YYLTYPE &loc )
-:
-	line(loc.first_line), 
-	col(loc.first_column)
+void Token::set( char *str, int len )
 {
+	length = len;
+	data = new char[len+1];
+	memcpy( data, str, len );
+	data[len] = 0;
 }
 
-void TokenData::append( const TokenData &other )
+void Token::append( const Token &other )
 {
 	int newLength = length + other.length;
 	char *newString = new char[newLength+1];
@@ -54,14 +56,6 @@ void TokenData::append( const TokenData &other )
 	length = newLength;
 }
 
-void TokenData::init( char c )
-{
-	data = new char[2];
-	data[0] = c;
-	data[1] = 0;
-	length = 1;
-}
-
 /* Perform minimization after an operation according 
  * to the command line args. */
 void afterOpMinimize( FsmAp *fsm, bool lastInSeq )
@@ -422,7 +416,7 @@ bool NameInst::anyRefsRec()
 
 /* Initialize the structure that will collect info during the parse of a
  * machine. */
-ParseData::ParseData( char *fileName, char *fsmName, 
+ParseData::ParseData( char *fileName, char *sectionName, 
 		const InputLoc &sectionLoc )
 :	
 	sectionGraph(0),
@@ -431,19 +425,28 @@ ParseData::ParseData( char *fileName, char *fsmName,
 	/* 0 is reserved for global error actions. */
 	nextLocalErrKey(1),
 	nextNameId(0),
+	nextCondId(0),
 	alphTypeSet(false),
 	getKeyExpr(0),
 	accessExpr(0),
-	curStateExpr(0),
+	pExpr(0),
+	peExpr(0),
+	csExpr(0),
+	topExpr(0),
+	stackExpr(0),
+	actExpr(0),
+	tokstartExpr(0),
+	tokendExpr(0),
+	dataExpr(0),
 	lowerNum(0),
 	upperNum(0),
 	fileName(fileName),
-	fsmName(fsmName),
+	sectionName(sectionName),
 	sectionLoc(sectionLoc),
-	errorCount(0),
 	curActionOrd(0),
 	curPriorOrd(0),
 	rootName(0),
+	exportsRootName(0),
 	nextEpsilonResolvedLink(0),
 	nextLongestMatchId(1),
 	lmRequiresErrorState(false)
@@ -481,6 +484,12 @@ void ParseData::initNameWalk()
 	curNameChild = 0;
 }
 
+void ParseData::initExportsNameWalk()
+{
+	curNameInst = exportsRootName;
+	curNameChild = 0;
+}
+
 /* Goes into the next child scope. The number of the child is already set up.
  * We need this for the syncronous name tree and parse tree walk to work
  * properly. It is reset on entry into a scope and advanced on poping of a
@@ -539,6 +548,7 @@ void ParseData::unsetObsoleteEntries( FsmAp *graph )
 		if ( name->numUses == name->numRefs ) {
 			assert( graph->entryPoints.find( name->id ) != 0 );
 			graph->unsetEntry( name->id );
+			assert( graph->entryPoints.find( name->id ) == 0 );
 		}
 	}
 }
@@ -773,18 +783,18 @@ void ParseData::fillNameIndex( NameInst *from )
 		fillNameIndex( *name );
 }
 
-void ParseData::makeRootName()
+void ParseData::makeRootNames()
 {
 	/* Create the root name. */
 	rootName = new NameInst( InputLoc(), 0, 0, nextNameId++, false );
+	exportsRootName = new NameInst( InputLoc(), 0, 0, nextNameId++, false );
 }
 
 /* Build the name tree and supporting data structures. */
 void ParseData::makeNameTree( GraphDictEl *dictEl )
 {
 	/* Set up curNameInst for the walk. */
-	curNameInst = rootName;
-	curNameChild = 0;
+	initNameWalk();
 
 	if ( dictEl != 0 ) {
 		/* A start location has been specified. */
@@ -802,8 +812,10 @@ void ParseData::makeNameTree( GraphDictEl *dictEl )
 	nameIndex = new NameInst*[nextNameId];
 	memset( nameIndex, 0, sizeof(NameInst*)*nextNameId );
 	fillNameIndex( rootName );
+	fillNameIndex( exportsRootName );
 }
 
+
 void ParseData::createBuiltin( char *name, BuiltinMachine builtin )
 {
 	Expression *expression = new Expression( builtin );
@@ -839,38 +851,45 @@ void ParseData::initGraphDict( )
 /* Set the alphabet type. If the types are not valid returns false. */
 bool ParseData::setAlphType( char *s1, char *s2 )
 {
-	bool valid = false;
-	for ( int i = 0; i < hostLang->numHostTypes; i++ ) {
-		if ( strcmp( s1, hostLang->hostTypes[i].data1 ) == 0 && 
-				hostLang->hostTypes[i].data2 != 0 && 
-				strcmp( s2, hostLang->hostTypes[i].data2 ) == 0 )
-		{
-			valid = true;
-			userAlphType = hostLang->hostTypes + i;
-			break;
-		}
-	}
-
+	userAlphType = findAlphType( s1, s2 );
 	alphTypeSet = true;
-	return valid;
+	return userAlphType != 0;
 }
 
 /* Set the alphabet type. If the types are not valid returns false. */
 bool ParseData::setAlphType( char *s1 )
 {
-	bool valid = false;
-	for ( int i = 0; i < hostLang->numHostTypes; i++ ) {
-		if ( strcmp( s1, hostLang->hostTypes[i].data1 ) == 0 && 
-				hostLang->hostTypes[i].data2 == 0 )
-		{
-			valid = true;
-			userAlphType = hostLang->hostTypes + i;
-			break;
-		}
-	}
-
+	userAlphType = findAlphType( s1 );
 	alphTypeSet = true;
-	return valid;
+	return userAlphType != 0;
+}
+
+bool ParseData::setVariable( char *var, InlineList *inlineList )
+{
+	bool set = true;
+
+	if ( strcmp( var, "p" ) == 0 )
+		pExpr = inlineList;
+	else if ( strcmp( var, "pe" ) == 0 )
+		peExpr = inlineList;
+	else if ( strcmp( var, "cs" ) == 0 )
+		csExpr = inlineList;
+	else if ( strcmp( var, "data" ) == 0 )
+		dataExpr = inlineList;
+	else if ( strcmp( var, "top" ) == 0 )
+		topExpr = inlineList;
+	else if ( strcmp( var, "stack" ) == 0 )
+		stackExpr = inlineList;
+	else if ( strcmp( var, "act" ) == 0 )
+		actExpr = inlineList;
+	else if ( strcmp( var, "tokstart" ) == 0 )
+		tokstartExpr = inlineList;
+	else if ( strcmp( var, "tokend" ) == 0 )
+		tokendExpr = inlineList;
+	else
+		set = false;
+
+	return set;
 }
 
 /* Initialize the key operators object that will be referenced by all fsms
@@ -897,7 +916,8 @@ void ParseData::printNameInst( NameInst *nameInst, int level )
 		cerr << "  ";
 	cerr << (nameInst->name != 0 ? nameInst->name : "<ANON>") << 
 			"  id: " << nameInst->id << 
-			"  refs: " << nameInst->numRefs << endl;
+			"  refs: " << nameInst->numRefs <<
+			"  uses: " << nameInst->numUses << endl;
 	for ( NameVect::Iter name = nameInst->childVect; name.lte(); name++ )
 		printNameInst( *name, level+1 );
 }
@@ -936,7 +956,12 @@ void ParseData::removeActionDups( FsmAp *graph )
 
 Action *ParseData::newAction( char *name, InlineList *inlineList )
 {
-	Action *action = new Action( InputLoc(1,1), name, inlineList, NameRefList() );
+	InputLoc loc;
+	loc.line = 1;
+	loc.col = 1;
+	loc.fileName = "<NONE>";
+
+	Action *action = new Action( loc, name, inlineList, nextCondId++ );
 	action->actionRefs.append( rootName );
 	actionList.append( action );
 	return action;
@@ -1105,6 +1130,8 @@ FsmAp *ParseData::makeSpecific( GraphDictEl *gdNode )
 	 * is okay since generating part of the graph is usually only done when
 	 * inspecting the compiled machine. */
 
+	/* Same story for extern entry point references. */
+
 	/* Flag this case so that the XML code generator is aware that we haven't
 	 * looked up name references in actions. It can then avoid segfaulting. */
 	generatingSectionSubset = true;
@@ -1129,6 +1156,10 @@ FsmAp *ParseData::makeAll()
 	/* Resolve action code name references. */
 	resolveActionNameRefs();
 
+	/* Force name references to the top level instantiations. */
+	for ( NameVect::Iter inst = rootName->childVect; inst.lte(); inst++ )
+		(*inst)->numRefs += 1;
+
 	FsmAp *mainGraph = 0;
 	FsmAp **graphs = new FsmAp*[instanceList.length()];
 	int numOthers = 0;
@@ -1136,23 +1167,19 @@ FsmAp *ParseData::makeAll()
 	/* Make all the instantiations, we know that main exists in this list. */
 	initNameWalk();
 	for ( GraphList::Iter glel = instanceList; glel.lte();  glel++ ) {
-		if ( strcmp( glel->key, machineMain ) == 0 ) {
+		if ( strcmp( glel->key, mainMachine ) == 0 ) {
 			/* Main graph is always instantiated. */
 			mainGraph = makeInstance( glel );
 		}
 		else {
-			/* Check to see if the instance is ever referenced. */
-			NameInst *nameInst = nextNameScope();
-			if ( nameInst->anyRefsRec() )
-				graphs[numOthers++] = makeInstance( glel );
-			else {
-				/* Need to walk over the name tree item. */
-				NameFrame nameFrame = enterNameScope( true, 1 );
-				popNameScope( nameFrame );
-			}
+			/* Instantiate and store in others array. */
+			graphs[numOthers++] = makeInstance( glel );
 		}
 	}
 
+	if ( mainGraph == 0 )
+		mainGraph = graphs[--numOthers];
+
 	if ( numOthers > 0 ) {
 		/* Add all the other graphs into main. */
 		mainGraph->globOp( graphs, numOthers );
@@ -1290,11 +1317,59 @@ void ParseData::analyzeGraph( FsmAp *graph )
 		checkAction( act );
 }
 
+void ParseData::makeExportsNameTree()
+{
+	/* Make a name tree for the exports. */
+	initExportsNameWalk();
+
+	/* First make the name tree. */
+	for ( GraphDict::Iter gdel = graphDict; gdel.lte(); gdel++ ) {
+		if ( gdel->value->isExport ) {
+			/* Recurse on the instance. */
+			gdel->value->makeNameTree( gdel->loc, this );
+		}
+	}
+}
+
+void ParseData::makeExports()
+{
+	makeExportsNameTree();
+
+	/* Resove name references in the tree. */
+	initExportsNameWalk();
+	for ( GraphDict::Iter gdel = graphDict; gdel.lte(); gdel++ ) {
+		if ( gdel->value->isExport )
+			gdel->value->resolveNameRefs( this );
+	}
+
+	/* Make all the instantiations, we know that main exists in this list. */
+	initExportsNameWalk();
+	for ( GraphDict::Iter gdel = graphDict; gdel.lte();  gdel++ ) {
+		/* Check if this var def is an export. */
+		if ( gdel->value->isExport ) {
+			/* Build the graph from a walk of the parse tree. */
+			FsmAp *graph = gdel->value->walk( this );
+
+			/* Build the graph from a walk of the parse tree. */
+			if ( !graph->checkSingleCharMachine() ) {
+				error(gdel->loc) << "bad export machine, must define "
+						"a single character" << endl;
+			}
+			else {
+				/* Safe to extract the key and declare the export. */
+				Key exportKey = graph->startState->outList.head->lowKey;
+				exportList.append( new Export( gdel->value->name, exportKey ) );
+			}
+		}
+	}
+
+}
+
 void ParseData::prepareMachineGen( GraphDictEl *graphDictEl )
 {
 	beginProcessing();
 	initKeyOps();
-	makeRootName();
+	makeRootNames();
 	initLongestMatchData();
 
 	/* Make the graph, do minimization. */
@@ -1302,6 +1377,9 @@ void ParseData::prepareMachineGen( GraphDictEl *graphDictEl )
 		sectionGraph = makeAll();
 	else
 		sectionGraph = makeSpecific( graphDictEl );
+	
+	/* Compute exports from the export definitions. */
+	makeExports();
 
 	/* If any errors have occured in the input file then don't write anything. */
 	if ( gblErrorCount > 0 )
@@ -1311,6 +1389,24 @@ void ParseData::prepareMachineGen( GraphDictEl *graphDictEl )
 
 	/* Depends on the graph analysis. */
 	setLongestMatchData( sectionGraph );
+
+	/* Decide if an error state is necessary.
+	 *  1. There is an error transition
+	 *  2. There is a gap in the transitions
+	 *  3. The longest match operator requires it. */
+	if ( lmRequiresErrorState || sectionGraph->hasErrorTrans() )
+		sectionGraph->errState = sectionGraph->addState();
+
+	/* State numbers need to be assigned such that all final states have a
+	 * larger state id number than all non-final states. This enables the
+	 * first_final mechanism to function correctly. We also want states to be
+	 * ordered in a predictable fashion. So we first apply a depth-first
+	 * search, then do a stable sort by final state status, then assign
+	 * numbers. */
+
+	sectionGraph->depthFirstOrdering();
+	sectionGraph->sortStatesByFinal();
+	sectionGraph->setStateNumbers( 0 );
 }
 
 void ParseData::generateXML( ostream &out )
@@ -1318,41 +1414,39 @@ void ParseData::generateXML( ostream &out )
 	beginProcessing();
 
 	/* Make the generator. */
-	XMLCodeGen codeGen( fsmName, this, sectionGraph, out );
+	XMLCodeGen codeGen( sectionName, this, sectionGraph, out );
 
 	/* Write out with it. */
 	codeGen.writeXML();
 
 	if ( printStatistics ) {
-		cerr << "fsm name  : " << fsmName << endl;
+		cerr << "fsm name  : " << sectionName << endl;
 		cerr << "num states: " << sectionGraph->stateList.length() << endl;
 		cerr << endl;
 	}
 }
 
-void checkMachines( )
+/* Send eof to all parsers. */
+void terminateAllParsers( )
 {
-	for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
-		ParseData *pd = sect->value;
-		if ( pd->instanceList.length() > 0 ) {
-			/* There must be a main graph defined. */
-			/* No machine name. Need to have a main. Make sure it was given. */
-			GraphDictEl *mainEl = pd->graphDict.find( machineMain );
-			if ( mainEl == 0 ) {
-				error(pd->sectionLoc) << "main graph not defined in \"" << 
-						pd->fsmName << "\"" << endl;
-			}
-		}
-	}
+	/* FIXME: a proper token is needed here. Suppose we should use the
+	 * location of EOF in the last file that the parser was referenced in. */
+	InputLoc loc;
+	loc.fileName = "<EOF>";
+	loc.line = 0;
+	loc.col = 0;
+	for ( ParserDict::Iter pdel = parserDict; pdel.lte(); pdel++ )
+		pdel->value->token( loc, _eof, 0, 0 );
 }
 
 void writeLanguage( std::ostream &out )
 {
 	out << " lang=\"";
-	switch ( hostLangType ) {
-		case CCode:    out << "C"; break;
-		case DCode:    out << "D"; break;
-		case JavaCode: out << "Java"; break;
+	switch ( hostLang->lang ) {
+		case HostLang::C:    out << "C"; break;
+		case HostLang::D:    out << "D"; break;
+		case HostLang::Java: out << "Java"; break;
+		case HostLang::Ruby: out << "Ruby"; break;
 	}
 	out << "\"";
 	
@@ -1362,18 +1456,18 @@ void writeMachines( std::ostream &out, std::string hostData, char *inputFileName
 {
 	if ( machineSpec == 0 && machineName == 0 ) {
 		/* No machine spec or machine name given. Generate everything. */
-		for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
-			ParseData *pd = sect->value;
+		for ( ParserDict::Iter parser = parserDict; parser.lte(); parser++ ) {
+			ParseData *pd = parser->value->pd;
 			if ( pd->instanceList.length() > 0 )
 				pd->prepareMachineGen( 0 );
 		}
 
 		if ( gblErrorCount == 0 ) {
-			out << "<ragel filename=\"" << inputFileName << "\"";
+			out << "<ragel version=\"" VERSION "\" filename=\"" << inputFileName << "\"";
 			writeLanguage( out );
 			out << ">\n";
-			for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
-				ParseData *pd = sect->value;
+			for ( ParserDict::Iter parser = parserDict; parser.lte(); parser++ ) {
+				ParseData *pd = parser->value->pd;
 				if ( pd->instanceList.length() > 0 )
 					pd->generateXML( out );
 			}
@@ -1381,16 +1475,16 @@ void writeMachines( std::ostream &out, std::string hostData, char *inputFileName
 			out << "</ragel>\n";
 		}
 	}
-	else if ( sectionMap.length() > 0 ) {
+	else if ( parserDict.length() > 0 ) {
 		/* There is either a machine spec or machine name given. */
 		ParseData *parseData = 0;
 		GraphDictEl *graphDictEl = 0;
 
 		/* Traverse the sections, break out when we find a section/machine
 		 * that matches the one specified. */
-		for ( SectionMap::Iter sect = sectionMap; sect.lte(); sect++ ) {
-			ParseData *checkPd = sect->value;
-			if ( machineSpec == 0 || strcmp( checkPd->fsmName, machineSpec ) == 0 ) {
+		for ( ParserDict::Iter parser = parserDict; parser.lte(); parser++ ) {
+			ParseData *checkPd = parser->value->pd;
+			if ( machineSpec == 0 || strcmp( checkPd->sectionName, machineSpec ) == 0 ) {
 				GraphDictEl *checkGdEl = 0;
 				if ( machineName == 0 || (checkGdEl = 
 						checkPd->graphDict.find( machineName )) != 0 )
@@ -1410,7 +1504,7 @@ void writeMachines( std::ostream &out, std::string hostData, char *inputFileName
 			/* Section/Machine to emit was found. Prepare and emit it. */
 			parseData->prepareMachineGen( graphDictEl );
 			if ( gblErrorCount == 0 ) {
-				out << "<ragel filename=\"" << inputFileName << "\"";
+				out << "<ragel version=\"" VERSION "\" filename=\"" << inputFileName << "\"";
 				writeLanguage( out );
 				out << ">\n";
 				parseData->generateXML( out );
