@@ -28,13 +28,62 @@
 /* Parsing. */
 #include "ragel.h"
 #include "rlparse.h"
-#include "parsedata.h"
 #include "parsetree.h"
 
 using namespace std;
 ostream &operator<<( ostream &out, const NameRef &nameRef );
 ostream &operator<<( ostream &out, const NameInst &nameInst );
 
+/* Convert the literal string which comes in from the scanner into an array of
+ * characters with escapes and options interpreted. Also null terminates the
+ * string. Though this null termination should not be relied on for
+ * interpreting literals in the parser because the string may contain a
+ * literal string with \0 */
+void Token::prepareLitString( Token &result, bool &caseInsensitive )
+{
+	result.data = new char[this->length+1];
+	caseInsensitive = false;
+
+	char *src = this->data + 1;
+	char *end = this->data + this->length - 1;
+
+	while ( *end != '\'' && *end != '\"' ) {
+		if ( *end == 'i' )
+			caseInsensitive = true;
+		else {
+			error( this->loc ) << "literal string '" << *end << 
+					"' option not supported" << endl;
+		}
+		end -= 1;
+	}
+
+	char *dest = result.data;
+	int len = 0;
+	while ( src != end ) {
+		if ( *src == '\\' ) {
+			switch ( src[1] ) {
+			case '0': dest[len++] = '\0'; break;
+			case 'a': dest[len++] = '\a'; break;
+			case 'b': dest[len++] = '\b'; break;
+			case 't': dest[len++] = '\t'; break;
+			case 'n': dest[len++] = '\n'; break;
+			case 'v': dest[len++] = '\v'; break;
+			case 'f': dest[len++] = '\f'; break;
+			case 'r': dest[len++] = '\r'; break;
+			case '\n':  break;
+			default: dest[len++] = src[1]; break;
+			}
+			src += 2;
+		}
+		else {
+			dest[len++] = *src++;
+		}
+	}
+	result.length = len;
+	result.data[result.length] = 0;
+}
+
+
 FsmAp *VarDef::walk( ParseData *pd )
 {
 	/* We enter into a new name scope. */
@@ -63,7 +112,7 @@ FsmAp *VarDef::walk( ParseData *pd )
 	 * the graph. */
 	if ( pd->curNameInst->numRefs > 0 )
 		rtnVal->setEntry( pd->curNameInst->id, rtnVal->startState );
-	
+
 	/* Pop the name scope. */
 	pd->popNameScope( nameFrame );
 	return rtnVal;
@@ -99,7 +148,7 @@ void VarDef::resolveNameRefs( ParseData *pd )
 
 InputLoc LongestMatchPart::getLoc()
 { 
-	return action != 0 ? action->loc : join->loc;
+	return action != 0 ? action->loc : semiLoc;
 }
 
 /*
@@ -115,7 +164,7 @@ InputLoc LongestMatchPart::getLoc()
 Action *LongestMatch::newAction( ParseData *pd, const InputLoc &loc, 
 		char *name, InlineList *inlineList )
 {
-	Action *action = new Action( loc, name, inlineList, NameRefList() );
+	Action *action = new Action( loc, name, inlineList, pd->nextCondId++ );
 	action->actionRefs.append( pd->curNameInst );
 	pd->actionList.append( action );
 	action->isLmAction = true;
@@ -143,7 +192,7 @@ void LongestMatch::makeActions( ParseData *pd )
 		inlineList->append( new InlineItem( lmi->getLoc(), this, lmi, 
 				InlineItem::LmOnLast ) );
 		char *actName = new char[50];
-		sprintf( actName, "imm%i", lmi->longestMatchId );
+		sprintf( actName, "last%i", lmi->longestMatchId );
 		lmi->actOnLast = newAction( pd, lmi->getLoc(), actName, inlineList );
 	}
 
@@ -157,7 +206,7 @@ void LongestMatch::makeActions( ParseData *pd )
 		inlineList->append( new InlineItem( lmi->getLoc(), this, lmi, 
 				InlineItem::LmOnNext ) );
 		char *actName = new char[50];
-		sprintf( actName, "lagh%i", lmi->longestMatchId );
+		sprintf( actName, "next%i", lmi->longestMatchId );
 		lmi->actOnNext = newAction( pd, lmi->getLoc(), actName, inlineList );
 	}
 
@@ -174,10 +223,14 @@ void LongestMatch::makeActions( ParseData *pd )
 		lmi->actLagBehind = newAction( pd, lmi->getLoc(), actName, inlineList );
 	}
 
+	InputLoc loc;
+	loc.line = 1;
+	loc.col = 1;
+
 	/* Create the error action. */
 	InlineList *il6 = new InlineList;
-	il6->append( new InlineItem( InputLoc(), this, 0, InlineItem::LmSwitch ) );
-	lmActSelect = newAction( pd, InputLoc(1,1), "lagsel", il6 );
+	il6->append( new InlineItem( loc, this, 0, InlineItem::LmSwitch ) );
+	lmActSelect = newAction( pd, loc, "switch", il6 );
 }
 
 void LongestMatch::findName( ParseData *pd )
@@ -293,6 +346,7 @@ void LongestMatch::runLonestMatch( ParseData *pd, FsmAp *graph )
 	}
 
 	/* The actions executed on starting to match a token. */
+	graph->isolateStartState();
 	graph->startState->toStateActionTable.setAction( pd->initTokStartOrd, pd->initTokStart );
 	graph->startState->fromStateActionTable.setAction( pd->setTokStartOrd, pd->setTokStart );
 	if ( maxItemSetLength > 1 ) {
@@ -1066,14 +1120,14 @@ void FactorWithAug::assignConditions( FsmAp *graph )
 		switch ( conditions[i].type ) {
 		/* Transition actions. */
 		case at_start:
-			graph->startFsmCondition( conditions[i].action );
+			graph->startFsmCondition( conditions[i].action, conditions[i].sense );
 			afterOpMinimize( graph );
 			break;
 		case at_all:
-			graph->allTransCondition( conditions[i].action );
+			graph->allTransCondition( conditions[i].action, conditions[i].sense );
 			break;
 		case at_leave:
-			graph->leaveFsmCondition( conditions[i].action );
+			graph->leaveFsmCondition( conditions[i].action, conditions[i].sense );
 			break;
 		default:
 			break;
@@ -1119,8 +1173,10 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 			actionOrd[i] = pd->curActionOrd++;
 	}
 
+	/* Embed conditions. */
 	assignConditions( rtnVal );
 
+	/* Embed actions. */
 	assignActions( pd, rtnVal , actionOrd );
 
 	/* Make the array of priority orderings. Orderings are local to this walk
@@ -1647,7 +1703,7 @@ Factor::~Factor()
 			delete reItem;
 			break;
 		case RegExprType:
-			delete regExp;
+			delete regExpr;
 			break;
 		case ReferenceType:
 			break;
@@ -1675,7 +1731,7 @@ FsmAp *Factor::walk( ParseData *pd )
 		rtnVal = reItem->walk( pd, 0 );
 		break;
 	case RegExprType:
-		rtnVal = regExp->walk( pd, 0 );
+		rtnVal = regExpr->walk( pd, 0 );
 		break;
 	case ReferenceType:
 		rtnVal = varDef->walk( pd );
@@ -1741,13 +1797,30 @@ Range::~Range()
 /* Evaluate a range. Gets the lower an upper key and makes an fsm range. */
 FsmAp *Range::walk( ParseData *pd )
 {
-	Key lowKey = lowerLit->makeRangeEndPoint( pd );
-	Key highKey = upperLit->makeRangeEndPoint( pd );
+	/* Construct and verify the suitability of the lower end of the range. */
+	FsmAp *lowerFsm = lowerLit->walk( pd );
+	if ( !lowerFsm->checkSingleCharMachine() ) {
+		error(lowerLit->token.loc) << 
+			"bad range lower end, must be a single character" << endl;
+	}
+
+	/* Construct and verify the upper end. */
+	FsmAp *upperFsm = upperLit->walk( pd );
+	if ( !upperFsm->checkSingleCharMachine() ) {
+		error(upperLit->token.loc) << 
+			"bad range upper end, must be a single character" << endl;
+	}
+
+	/* Grab the keys from the machines, then delete them. */
+	Key lowKey = lowerFsm->startState->outList.head->lowKey;
+	Key highKey = upperFsm->startState->outList.head->lowKey;
+	delete lowerFsm;
+	delete upperFsm;
 
 	/* Validate the range. */
 	if ( lowKey > highKey ) {
 		/* Recover by setting upper to lower; */
-		error(lowerLit->loc) << "lower end of range is greater then upper end" << endl;
+		error(lowerLit->token.loc) << "lower end of range is greater then upper end" << endl;
 		highKey = lowKey;
 	}
 
@@ -1757,24 +1830,6 @@ FsmAp *Range::walk( ParseData *pd )
 	return retFsm;
 }
 
-/* Return an fsm key for a literal that is a range end point. */
-Key Literal::makeRangeEndPoint( ParseData *pd )
-{
-	Key retVal;
-	switch ( type ) {
-	case Number: {
-		/* Make the fsm key from the number. */
-		retVal = makeFsmKeyNum( data.data, loc, pd );
-		break;
-	}
-	case LitString: {
-		/* The string should be verified length one. */
-		makeFsmKeyArray( &retVal, data.data, 1, pd );
-		break;
-	}}
-	return retVal;
-}
-
 /* Evaluate a literal object. */
 FsmAp *Literal::walk( ParseData *pd )
 {
@@ -1784,7 +1839,7 @@ FsmAp *Literal::walk( ParseData *pd )
 	switch ( type ) {
 	case Number: {
 		/* Make the fsm key in int format. */
-		Key fsmKey = makeFsmKeyNum( data.data, loc, pd );
+		Key fsmKey = makeFsmKeyNum( token.data, token.loc, pd );
 		/* Make the new machine. */
 		rtnVal = new FsmAp();
 		rtnVal->concatFsm( fsmKey );
@@ -1792,15 +1847,19 @@ FsmAp *Literal::walk( ParseData *pd )
 	}
 	case LitString: {
 		/* Make the array of keys in int format. */
-		Key *arr = new Key[data.length];
-		makeFsmKeyArray( arr, data.data, data.length, pd );
+		Token interp;
+		bool caseInsensitive;
+		token.prepareLitString( interp, caseInsensitive );
+		Key *arr = new Key[interp.length];
+		makeFsmKeyArray( arr, interp.data, interp.length, pd );
 
 		/* Make the new machine. */
 		rtnVal = new FsmAp();
 		if ( caseInsensitive )
-			rtnVal->concatFsmCI( arr, data.length );
+			rtnVal->concatFsmCI( arr, interp.length );
 		else
-			rtnVal->concatFsm( arr, data.length );
+			rtnVal->concatFsm( arr, interp.length );
+		delete[] interp.data;
 		delete[] arr;
 		break;
 	}}
@@ -1812,7 +1871,7 @@ RegExpr::~RegExpr()
 {
 	switch ( type ) {
 		case RecurseItem:
-			delete regExp;
+			delete regExpr;
 			delete item;
 			break;
 		case Empty:
@@ -1831,18 +1890,14 @@ FsmAp *RegExpr::walk( ParseData *pd, RegExpr *rootRegex )
 	switch ( type ) {
 		case RecurseItem: {
 			/* Walk both items. */
-			FsmAp *fsm1 = regExp->walk( pd, rootRegex );
+			rtnVal = regExpr->walk( pd, rootRegex );
 			FsmAp *fsm2 = item->walk( pd, rootRegex );
-			if ( fsm1 == 0 )
-				rtnVal = fsm2;
-			else {
-				fsm1->concatOp( fsm2 );
-				rtnVal = fsm1;
-			}
+			rtnVal->concatOp( fsm2 );
 			break;
 		}
 		case Empty: {
-			rtnVal = 0;
+			rtnVal = new FsmAp();
+			rtnVal->lambdaFsm();
 			break;
 		}
 	}
@@ -1872,15 +1927,15 @@ FsmAp *ReItem::walk( ParseData *pd, RegExpr *rootRegex )
 	switch ( type ) {
 		case Data: {
 			/* Move the data into an integer array and make a concat fsm. */
-			Key *arr = new Key[data.length];
-			makeFsmKeyArray( arr, data.data, data.length, pd );
+			Key *arr = new Key[token.length];
+			makeFsmKeyArray( arr, token.data, token.length, pd );
 
 			/* Make the concat fsm. */
 			rtnVal = new FsmAp();
 			if ( rootRegex != 0 && rootRegex->caseInsensitive )
-				rtnVal->concatFsmCI( arr, data.length );
+				rtnVal->concatFsmCI( arr, token.length );
 			else
-				rtnVal->concatFsm( arr, data.length );
+				rtnVal->concatFsm( arr, token.length );
 			delete[] arr;
 			break;
 		}
@@ -1892,6 +1947,10 @@ FsmAp *ReItem::walk( ParseData *pd, RegExpr *rootRegex )
 		case OrBlock: {
 			/* Get the or block and minmize it. */
 			rtnVal = orBlock->walk( pd, rootRegex );
+			if ( rtnVal == 0 ) {
+				rtnVal = new FsmAp();
+				rtnVal->lambdaFsm();
+			}
 			rtnVal->minimizePartition2();
 			break;
 		}
@@ -1975,7 +2034,7 @@ FsmAp *ReOrItem::walk( ParseData *pd, RegExpr *rootRegex )
 		 * issue warning or an error but since we can't with [a0-9a] or 'a' |
 		 * 'a' don't bother here. */
 		KeySet keySet;
-		makeFsmUniqueKeyArray( keySet, data.data, data.length, 
+		makeFsmUniqueKeyArray( keySet, token.data, token.length, 
 			rootRegex != 0 ? rootRegex->caseInsensitive : false, pd );
 
 		/* Run the or operator. */
