@@ -22,12 +22,197 @@
 #ifndef _PARSETREE_H
 #define _PARSETREE_H
 
+#include "ragel.h"
 #include "avlmap.h"
 #include "bstmap.h"
 #include "vector.h"
 #include "dlist.h"
-#include "rlparse.h"
-#include "parsedata.h"
+
+struct NameInst;
+
+/* Types of builtin machines. */
+enum BuiltinMachine
+{
+	BT_Any,
+	BT_Ascii,
+	BT_Extend,
+	BT_Alpha,
+	BT_Digit,
+	BT_Alnum,
+	BT_Lower,
+	BT_Upper,
+	BT_Cntrl,
+	BT_Graph,
+	BT_Print,
+	BT_Punct,
+	BT_Space,
+	BT_Xdigit,
+	BT_Lambda,
+	BT_Empty
+};
+
+
+struct ParseData;
+
+/* Leaf type. */
+struct Literal;
+
+/* Tree nodes. */
+
+struct Term;
+struct FactorWithAug;
+struct FactorWithRep;
+struct FactorWithNeg;
+struct Factor;
+struct Expression;
+struct Join;
+struct JoinOrLm;
+struct LongestMatch;
+struct LongestMatchPart;
+struct LmPartList;
+struct Range;
+
+/* Type of augmentation. Describes locations in the machine. */
+enum AugType
+{
+	/* Transition actions/priorities. */
+	at_start,
+	at_all,
+	at_finish,
+	at_leave,
+
+	/* Global error actions. */
+	at_start_gbl_error,
+	at_all_gbl_error,
+	at_final_gbl_error,
+	at_not_start_gbl_error,
+	at_not_final_gbl_error,
+	at_middle_gbl_error,
+
+	/* Local error actions. */
+	at_start_local_error,
+	at_all_local_error,
+	at_final_local_error,
+	at_not_start_local_error,
+	at_not_final_local_error,
+	at_middle_local_error,
+	
+	/* To State Action embedding. */
+	at_start_to_state,
+	at_all_to_state,
+	at_final_to_state,
+	at_not_start_to_state,
+	at_not_final_to_state,
+	at_middle_to_state,
+
+	/* From State Action embedding. */
+	at_start_from_state,
+	at_all_from_state,
+	at_final_from_state,
+	at_not_start_from_state,
+	at_not_final_from_state,
+	at_middle_from_state,
+
+	/* EOF Action embedding. */
+	at_start_eof,
+	at_all_eof,
+	at_final_eof,
+	at_not_start_eof,
+	at_not_final_eof,
+	at_middle_eof
+};
+
+/* IMPORTANT: These must follow the same order as the state augs in AugType
+ * since we will be using this to compose AugType. */
+enum StateAugType
+{
+	sat_start = 0,
+	sat_all,
+	sat_final,
+	sat_not_start,
+	sat_not_final,
+	sat_middle
+};
+
+struct Action;
+struct PriorDesc;
+struct RegExpr;
+struct ReItem;
+struct ReOrBlock;
+struct ReOrItem;
+struct ExplicitMachine;
+struct InlineItem;
+struct InlineList;
+
+/* Reference to a named state. */
+typedef Vector<char*> NameRef;
+typedef Vector<NameRef*> NameRefList;
+typedef Vector<NameInst*> NameTargList;
+
+/* Structure for storing location of epsilon transitons. */
+struct EpsilonLink
+{
+	EpsilonLink( const InputLoc &loc, NameRef &target )
+		: loc(loc), target(target) { }
+
+	InputLoc loc;
+	NameRef target;
+};
+
+struct Label
+{
+	Label( const InputLoc &loc, char *data )
+		: loc(loc), data(data) { }
+
+	InputLoc loc;
+	char *data;
+};
+
+/* Structrue represents an action assigned to some FactorWithAug node. The
+ * factor with aug will keep an array of these. */
+struct ParserAction
+{
+	ParserAction( const InputLoc &loc, AugType type, int localErrKey, Action *action )
+		: loc(loc), type(type), localErrKey(localErrKey), action(action) { }
+
+	InputLoc loc;
+	AugType type;
+	int localErrKey;
+	Action *action;
+};
+
+struct ConditionTest
+{
+	ConditionTest( const InputLoc &loc, AugType type, Action *action, bool sense ) : 
+		loc(loc), type(type), action(action), sense(sense) { }
+
+	InputLoc loc;
+	AugType type;
+	Action *action;
+	bool sense;
+};
+
+struct Token
+{
+	char *data;
+	int length;
+	InputLoc loc;
+
+	void prepareLitString( Token &result, bool &caseInsensitive );
+	void append( const Token &other );
+	void set( char *str, int len );
+};
+
+/* Store the value and type of a priority augmentation. */
+struct PriorityAug
+{
+	PriorityAug( AugType type, int priorKey, int priorValue ) :
+		type(type), priorKey(priorKey), priorValue(priorValue) { }
+
+	AugType type;
+	int priorKey;
+	int priorValue;
+};
 
 /*
  * A Variable Definition
@@ -35,7 +220,7 @@
 struct VarDef
 {
 	VarDef( char *name, JoinOrLm *joinOrLm )
-		: name(name), joinOrLm(joinOrLm) { }
+		: name(name), joinOrLm(joinOrLm), isExport(false) { }
 	
 	/* Parse tree traversal. */
 	FsmAp *walk( ParseData *pd );
@@ -44,6 +229,7 @@ struct VarDef
 
 	char *name;
 	JoinOrLm *joinOrLm;
+	bool isExport;
 };
 
 
@@ -75,14 +261,18 @@ struct VarDef
  */
 struct LongestMatchPart
 {
-	LongestMatchPart( Join *join, Action *action, int longestMatchId ) : 
-		join(join), action(action), longestMatchId(longestMatchId),
-		inLmSelect(false) { }
+	LongestMatchPart( Join *join, Action *action, 
+			InputLoc &semiLoc, int longestMatchId )
+	: 
+		join(join), action(action), semiLoc(semiLoc), 
+		longestMatchId(longestMatchId), inLmSelect(false) { }
 
 	InputLoc getLoc();
 	
 	Join *join;
 	Action *action;
+	InputLoc semiLoc;
+
 	Action *setActId;
 	Action *actOnLast;
 	Action *actOnNext;
@@ -251,24 +441,6 @@ struct Term
 	PriorDesc priorDescs[2];
 };
 
-/* Structure for storing location of epsilon transitons. */
-struct EpsilonLink
-{
-	EpsilonLink( const InputLoc &loc, NameRef &target )
-		: loc(loc), target(target) { }
-
-	InputLoc loc;
-	NameRef target;
-};
-
-struct Label
-{
-	Label( const InputLoc &loc, char *data )
-		: loc(loc), data(data) { }
-
-	InputLoc loc;
-	char *data;
-};
 
 /* Third level of precedence. Augmenting nodes with actions and priorities. */
 struct FactorWithAug
@@ -293,7 +465,7 @@ struct FactorWithAug
 	PriorDesc *priorDescs;
 	Vector<Label> labels;
 	Vector<EpsilonLink> epsilonLinks;
-	Vector<ParserAction> conditions;
+	Vector<ConditionTest> conditions;
 
 	FactorWithRep *factorWithRep;
 };
@@ -320,8 +492,8 @@ struct FactorWithRep
 		factorWithNeg(0), lowerRep(lowerRep), 
 		upperRep(upperRep), type(type) { }
 	
-	FactorWithRep( const InputLoc &loc, FactorWithNeg *factorWithNeg )
-		: loc(loc), factorWithNeg(factorWithNeg), type(FactorWithNegType) { }
+	FactorWithRep( FactorWithNeg *factorWithNeg )
+		: factorWithNeg(factorWithNeg), type(FactorWithNegType) { }
 
 	~FactorWithRep();
 
@@ -352,8 +524,8 @@ struct FactorWithNeg
 	FactorWithNeg( const InputLoc &loc, FactorWithNeg *factorWithNeg, Type type) :
 		loc(loc), factorWithNeg(factorWithNeg), factor(0), type(type) { }
 
-	FactorWithNeg( const InputLoc &loc, Factor *factor ) :
-		loc(loc), factorWithNeg(0), factor(factor), type(FactorType) { }
+	FactorWithNeg( Factor *factor ) :
+		factorWithNeg(0), factor(factor), type(FactorType) { }
 
 	~FactorWithNeg();
 
@@ -397,8 +569,8 @@ struct Factor
 		reItem(reItem), type(OrExprType) { }
 
 	/* Construct with a regular expression. */
-	Factor( RegExpr *regExp ) :
-		regExp(regExp), type(RegExprType) { }
+	Factor( RegExpr *regExpr ) :
+		regExpr(regExpr), type(RegExprType) { }
 
 	/* Construct with a reference to a var def. */
 	Factor( const InputLoc &loc, VarDef *varDef ) :
@@ -424,7 +596,7 @@ struct Factor
 	Literal *literal;
 	Range *range;
 	ReItem *reItem;
-	RegExpr *regExp;
+	RegExpr *regExpr;
 	VarDef *varDef;
 	Join *join;
 	LongestMatch *longestMatch;
@@ -450,16 +622,13 @@ struct Literal
 {
 	enum LiteralType { Number, LitString };
 
-	Literal( const InputLoc &loc, TokenData data, LiteralType type )
-		: loc(loc), data(data), type(type), caseInsensitive(false) { }
+	Literal( const Token &token, LiteralType type )
+		: token(token), type(type) { }
 
-	Key makeRangeEndPoint( ParseData *pd );
 	FsmAp *walk( ParseData *pd );
 	
-	InputLoc loc;
-	TokenData data;
+	Token token;
 	LiteralType type;
-	bool caseInsensitive;
 };
 
 /* Regular expression. */
@@ -470,14 +639,14 @@ struct RegExpr
 	/* Constructors. */
 	RegExpr() : 
 		type(Empty), caseInsensitive(false) { }
-	RegExpr(RegExpr *regExp, ReItem *item) : 
-		regExp(regExp), item(item), 
+	RegExpr(RegExpr *regExpr, ReItem *item) : 
+		regExpr(regExpr), item(item), 
 		type(RecurseItem), caseInsensitive(false) { }
 
 	~RegExpr();
 	FsmAp *walk( ParseData *pd, RegExpr *rootRegex );
 
-	RegExpr *regExp;
+	RegExpr *regExpr;
 	ReItem *item;
 	RegExpType type;
 	bool caseInsensitive;
@@ -488,11 +657,8 @@ struct ReItem
 {
 	enum ReItemType { Data, Dot, OrBlock, NegOrBlock };
 	
-	ReItem( const InputLoc &loc, char c ) 
-		: loc(loc), star(false), type(Data)
-	{
-		data.init( c );
-	}
+	ReItem( const InputLoc &loc, const Token &token ) 
+		: loc(loc), token(token), star(false), type(Data) { }
 	ReItem( const InputLoc &loc, ReItemType type )
 		: loc(loc), star(false), type(type) { }
 	ReItem( const InputLoc &loc, ReOrBlock *orBlock, ReItemType type )
@@ -502,7 +668,7 @@ struct ReItem
 	FsmAp *walk( ParseData *pd, RegExpr *rootRegex );
 
 	InputLoc loc;
-	TokenData data;
+	Token token;
 	ReOrBlock *orBlock;
 	bool star;
 	ReItemType type;
@@ -532,18 +698,15 @@ struct ReOrItem
 {
 	enum ReOrItemType { Data, Range };
 
-	ReOrItem( const InputLoc &loc, char c ) 
-		: loc(loc), type(Data)
-	{
-		data.init( c );
-	}
+	ReOrItem( const InputLoc &loc, const Token &token ) 
+		: loc(loc), token(token), type(Data) {}
 	ReOrItem( const InputLoc &loc, char lower, char upper )
 		: loc(loc), lower(lower), upper(upper), type(Range) { }
 
 	FsmAp *walk( ParseData *pd, RegExpr *rootRegex );
 
 	InputLoc loc;
-	TokenData data;
+	Token token;
 	char lower;
 	char upper;
 	ReOrItemType type;
@@ -598,4 +761,6 @@ struct InlineItem
  * ptreetypes, which should be just typedef forwards. */
 struct InlineList : public DList<InlineItem> { };
 
+
+
 #endif /* _PARSETREE_H */
