@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005, 2006 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2005-2007 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -90,27 +90,23 @@ private:
 
 	void writeKey( Key key );
 	void writeText( InlineItem *item );
-	void writeCtrlFlow( InlineItem *item, InlineItem *context );
-	void writePtrMod( InlineItem *item, InlineItem *context );
-	void writeGoto( InlineItem *item, InlineItem *context );
-	void writeGotoExpr( InlineItem *item, InlineItem *context );
-	void writeCall( InlineItem *item, InlineItem *context );
-	void writeCallExpr( InlineItem *item, InlineItem *context );
-	void writeNext( InlineItem *item, InlineItem *context );
-	void writeNextExpr( InlineItem *item, InlineItem *context );
+	void writeGoto( InlineItem *item );
+	void writeGotoExpr( InlineItem *item );
+	void writeCall( InlineItem *item );
+	void writeCallExpr( InlineItem *item );
+	void writeNext( InlineItem *item );
+	void writeNextExpr( InlineItem *item );
 	void writeEntry( InlineItem *item );
 	void writeLmSetActId( InlineItem *item );
 	void writeLmOnLast( InlineItem *item );
 	void writeLmOnNext( InlineItem *item );
 	void writeLmOnLagBehind( InlineItem *item );
 
+	void writeExports();
+	bool writeNameInst( NameInst *nameInst );
 	void writeEntryPoints();
-	void writeGetKeyExpr();
-	void writeAccessExpr();
-	void writeCurStateExpr();
 	void writeConditions();
-	void writeInlineList( InlineList *inlineList, InlineItem *context );
-	void writeAlphType();
+	void writeInlineList( InlineList *inlineList );
 	void writeActionList();
 	void writeActionTableList();
 	void reduceTrans( TransAp *trans );
@@ -121,7 +117,6 @@ private:
 	void writeLmSwitch( InlineItem *item );
 	void writeMachine();
 	void writeActionExec( InlineItem *item );
-	void writeActionExecTE( InlineItem *item );
 
 	char *fsmName;
 	ParseData *pd;
